<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ConfiguracaoCardapio extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $utilitiesCard;

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $utilitiesCard) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->utilitiesCard = $utilitiesCard;
	}
	
	public function saveConfigCard(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    $this->connection->beginTransaction();
	    try {
	    	foreach($row['editedRowsCard'] as $editedRows) {
	    		if ($editedRows['__is_new']) {
	    			$this->insertNewRegisterIntoConfiCard($row['filterData'], $editedRows);
	    		} else {
	    			$this->updatePlates($editedRows);
	    			$this->updateRegisterIntoConfiCard($editedRows);
	    		}
	    	}
	    	
	    	$this->connection->commit();
	    	$response->addDataSet(new DataSet("saveConfigCard", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function getMaxSeqConfCard(DTO\Request\Row $request, DTO\Response $response) {
		$sql = "SELECT MAX(NRSEQCNFCPME) AS NRSEQCNFCPME, MAX(NRORDCNFCPME) AS NRORDCNFCPME FROM CONFCARDPME WHERE NRCARDPME = :NRCARDPME AND CDSERVFILI = :CDSERVFILI AND CDFILCENTPME = :CDFILCENTPME";
		$row = $request->getRow();
		$paramSelect = array(
            "NRCARDPME"	    => $row['NRCARDPME'],
        	"CDSERVFILI"	=> $row['service']['CDSERVFILI'],
            "CDFILCENTPME"  => $row['cdfilcentpme']
        );
      
      $maxSeq = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
      $response->addDataSet(new DataSet("maxSeq", $maxSeq));
	}
	
	private function insertNewRegisterIntoConfiCard($filterData, $row) {
		$insertConfig = "INSERT INTO CONFCARDPME (CDFILCENTPME, NRCARDPME, CDSERVFILI, NRSEQCNFCPME, CDTIPOPRAT, IDOPCCNFCPME, IDIMPCNFCPME, NRORDCNFCPME) VALUES (:CDFILCENTPME, :NRCARDPME, :CDSERVFILI, :NRSEQCNFCPME, :CDTIPOPRAT, :IDOPCCNFCPME, :IDIMPCNFCPME, :NRORDCNFCPME)";

		if ($row['NRSEQCNFCPME'] > 999) {
			$row['NRSEQCNFCPME'] = $this->getPossibleSequentialConfCard($filterData);
		}

		$params = array(
            "CDFILCENTPME"   => $filterData['cdfilcentpme'],
            "NRCARDPME"      => $filterData['NRCARDPME'],
            "CDSERVFILI"	 => $filterData['service']['CDSERVFILI'],
            "NRSEQCNFCPME"	 => $row['NRSEQCNFCPME'],
            "CDTIPOPRAT"	 => $row['DESCRICAO'], // DESCRICAO É O CDTIPOPRAT NO DRAGANDDROP QUE VAI SER ADD.
            "IDOPCCNFCPME"   => $row['IDOPCCNFCPME'],
            "IDIMPCNFCPME"   => $row['IDIMPCNFCPME'],
            "NRORDCNFCPME"	 => $row['NRORDCNFCPME']
        );
        
        $stmt = $this->connection->prepare($insertConfig);
    	$stmt->execute($params);
	}
	
	// ACHA O INTERVALO DE SEQUENCIAL POSSÍVEL PARA INSERÇÃO
	private function getPossibleSequentialConfCard($row) {
		$sql = "SELECT NRSEQCNFCPME FROM CONFCARDPME WHERE NRCARDPME = :NRCARDPME AND CDSERVFILI = :CDSERVFILI AND CDFILCENTPME = :CDFILCENTPME
			ORDER BY NRSEQCNFCPME";
		
		$paramSelect = array(
            "NRCARDPME"	    => $row['NRCARDPME'],
        	"CDSERVFILI"	=> $row['service']['CDSERVFILI'],
            "CDFILCENTPME"  => $row['cdfilcentpme']
        );
      
    	$result = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
    	$possibleSequential = '';
		$aux = null;
    	foreach($result as $conf) {
    		if ($aux === null) {
    			$aux = $conf['NRSEQCNFCPME'];
    			continue;
    		}
    		if (($aux + 1) !== $conf['NRSEQCNFCPME']) {
    			$possibleSequential = $aux + 1;
    			break;
    		} else $aux = $conf['NRSEQCNFCPME'];
    	}
    	if ($possibleSequential === '') $possibleSequential = 1;
    	return $possibleSequential;
	}
	
	private function updatePlates($editedRows) {
		$tpPlates = $this->getTpPlateCardia($editedRows);
		$updatePlates = "UPDATE COMPCARDPME SET NRORDPRACPME = :NRORDCNFCPME WHERE CDFILCENTPME = :CDFILCENTPME AND NRCARDPME = :NRCARDPME AND NRCARDIAPME = :NRCARDIAPME AND NRSEQPRACPME = :NRSEQPRACPME AND CDTIPOPRAT = :CDTIPOPRAT";
		
		foreach($tpPlates as $plate) {
			$params = array(
				"NRORDCNFCPME"	=> $editedRows['NRORDCNFCPME'],
		        "CDFILCENTPME"  => $plate['CDFILCENTPME'],
	            "NRCARDPME"     => $plate['NRCARDPME'],
	            "NRCARDIAPME"   => $plate['NRCARDIAPME'],
	            "NRSEQPRACPME"	=> $plate['NRSEQPRACPME'],
	            "CDTIPOPRAT"	=> $plate['CDTIPOPRAT']
		    );
		    
		    $stmt = $this->connection->prepare($updatePlates);
			$stmt->execute($params);
		}
	}
	
	private function updateRegisterIntoConfiCard($row) {
		$updateConfig = "UPDATE CONFCARDPME SET NRORDCNFCPME = :NRORDCNFCPME, IDOPCCNFCPME = :IDOPCCNFCPME, IDIMPCNFCPME = :IDIMPCNFCPME WHERE CDFILCENTPME = :CDFILCENTPME AND NRCARDPME = :NRCARDPME AND CDSERVFILI = :CDSERVFILI AND NRSEQCNFCPME = :NRSEQCNFCPME AND CDTIPOPRAT = :CDTIPOPRAT";
		
		$params = array(
	        "CDFILCENTPME"   => $row['CDFILCENTPME'],
	        "NRCARDPME"      => $row['NRCARDPME'],
	        "CDSERVFILI"	 => $row['CDSERVFILI'],
	        "NRSEQCNFCPME"	 => $row['NRSEQCNFCPME'],
	        "CDTIPOPRAT"	 => $row['CDTIPOPRAT'],
	        "IDOPCCNFCPME"   => $row['IDOPCCNFCPME'],
	        "IDIMPCNFCPME"   => $row['IDIMPCNFCPME'],
	        "NRORDCNFCPME"	 => $row['NRORDCNFCPME']
	    );
	    
	    $stmt = $this->connection->prepare($updateConfig);
		$stmt->execute($params);
	}
	
	public function deleteConfCard(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		try {
			$this->connection->beginTransaction();
			foreach($row as $selectedRow) {
				$tpPlates = $this->getTpPlateCardia($selectedRow);
				if (count($tpPlates) > 0) $this->deleteTpPlates($tpPlates);
				$this->deleteRegisterConfigCard($selectedRow);
			}
			
			$this->connection->commit();
	    	$response->addDataSet(new DataSet("deleteConfigCard", array(true)));
		} catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getTpPlateCardia($selectedRow) {
		$sql = "SELECT C.CDFILCENTPME, C.CDSERVFILI, C.NRCARDIAPME, C.NRCARDPME, P.NRSEQPRACPME, P.CDTIPOPRAT, P.NRORDPRACPME FROM CARDIAPME C, COMPCARDPME P, CARDAPIOPME CA WHERE C.CDFILCENTPME  = :P_CDFILCENTPME AND C.CDSERVFILI    = :P_CDSERVFILI   AND C.NRCARDPME     = :P_NRCARDPME    AND CA.DTINICARDPME = :P_DTINICARDPME AND CA.DTFINCARDPME = :P_DTFINCARDPME AND CA.CDTIPOCARD   = :P_CDTIPOCARD   AND P.CDFILCENTPME  = C.CDFILCENTPME  AND P.CDFILCENTPME  = CA.CDFILCENTPME AND P.NRCARDIAPME   = C.NRCARDIAPME   AND P.NRCARDPME     = C.NRCARDPME     AND C.NRCARDPME     = CA.NRCARDPME    AND P.NRSEQPRACPME  = :P_NRSEQ";
		
		$paramSelect = array(
	        "P_CDFILCENTPME"  => $selectedRow['CDFILCENTPME'],
	    	"P_CDSERVFILI"	  => $selectedRow['CDSERVFILI'],
	        "P_NRCARDPME"     => $selectedRow['NRCARDPME'],
	        "P_DTINICARDPME"  => $selectedRow['DTINICARDPME'],
	        "P_DTFINCARDPME"  => $selectedRow['DTFINCARDPME'],
	        "P_CDTIPOCARD"    => $selectedRow['CDTIPOCARD'],
	        "P_NRSEQ"         => $selectedRow['NRSEQCNFCPME']
	    );
	    
    	return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function deleteTpPlates($tpPlates) {
		foreach($tpPlates as $plate) {
			$this->utilitiesCard->deleteCompCard($plate['CDFILCENTPME'], $plate['NRCARDPME'], $plate['NRCARDIAPME'], $plate['NRSEQPRACPME'], $plate['CDTIPOPRAT']);
		}
	}
	
	private function deleteRegisterConfigCard($selectedRow) {
		$delete = "DELETE FROM CONFCARDPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDSERVFILI = :OLD_CDSERVFILI AND NRSEQCNFCPME = :OLD_NRSEQCNFCPME AND NRCARDPME = :OLD_NRCARDPME";
		
		$params = array(
            "OLD_CDFILCENTPME" => $selectedRow['CDFILCENTPME'],
            "OLD_CDSERVFILI"   => $selectedRow['CDSERVFILI'],
            "OLD_NRSEQCNFCPME" => $selectedRow['NRSEQCNFCPME'],
            "OLD_NRCARDPME"	   => $selectedRow['NRCARDPME']
        );	
        
        $stmt = $this->connection->prepare($delete);
    	$stmt->execute($params);
	}
	
	public function saveImportTpPrato(DTO\Request\Row $request, DTO\Response $response){
		$parameters = $request->getParameters();
        $rows = $parameters['checkedRows'];
        $configOrig = '';
        
		foreach($rows as $row){
			$configOrig .= $row['NRSEQCNFCPME']."_"; 
		}
		
		
		//diferença de dias datas de origem
		$dtDe = str_replace("/", "-", $row['DTDE']);
		$dtAte = str_replace("/", "-", $row['DTATE']);
		$dtIniDest = str_replace("/", "-", $row['INCLUIRCARD']);
		$diferenca = strtotime($dtAte) - strtotime($dtDe);
		$qtdDias = $diferenca / (60*60*24);
		$dtFinDest = date('d/m/Y', strtotime("+{$qtdDias} days", strtotime($dtIniDest)));
		
        //dtFinDest não pode ultrapassar a data final do cardápio.
        if ($dtFinDest > $row['DATAFINALCARDAPIO']) $dtFinDest = $row['DATAFINALCARDAPIO'];

		$connection = $this->entityManager->getConnection();
		$procedure = new StoredProcedure($connection, 'EXPORTA_CARDAPIOPME');
		
		$procedure->addParam(new Param('P_CDFILIORIG', Param::PARAM_INPUT, $row['CDFILCENTPME'],     Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDSERVORIG', Param::PARAM_INPUT, $row['CDSERVFILI'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDTIPOCARD', Param::PARAM_INPUT, $row['CDTIPOCARD'],       Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRCARDORIG', Param::PARAM_INPUT, $row['NRCARDPME'],        Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRCARDDEST', Param::PARAM_INPUT, $row['NRCARDPMEDESTINO'], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINIORIG',  Param::PARAM_INPUT, $row['DTDE'],             Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINORIG',  Param::PARAM_INPUT, $row['DTATE'],            Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDFILIDEST', Param::PARAM_INPUT, $row['CENTRALDESTINO'],   Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDSERVDEST', Param::PARAM_INPUT, $row['SERVICODESTINO'],   Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTINIDEST',  Param::PARAM_INPUT, $row['INCLUIRCARD'],      Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DTFINDEST',  Param::PARAM_INPUT, $dtFinDest,               Param::PARAM_TYPE_STR)); 
		$procedure->addParam(new Param('P_CONFIGORIG', Param::PARAM_INPUT, $configOrig,              Param::PARAM_TYPE_STR));
		$procedure->execute();
	}
	
	public function returnDadosCardapioImportacao(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$query = "SELECT DISTINCT CA.CDFILCENTPME, CA.NRCARDPME, TO_CHAR(CA.DTINICARDPME, 'DD/MM/YYYY') DTINICARDPME, CA.DTINICARDPME DTINI, TO_CHAR(CA.DTFINCARDPME, 'DD/MM/YYYY') DTFINCARDPME, CA.DTFINCARDPME DTFIN, CA.DSCARDPME, TC.CDTIPOCARD, CF.CDSERVFILI FROM CARDAPIOPME CA, CONFCARDPME CF, TIPOCARD    TC WHERE (CA.CDFILCENTPME = :P_CDFILCENTPME) AND (CF.CDSERVFILI   = :P_CDSERVFILI)   AND (TC.CDTIPOCARD   = :P_CDTIPOCARD)   AND (CF.CDFILCENTPME = TC.CDFILCENTPME) AND (CF.CDFILCENTPME = CA.CDFILCENTPME) AND (CF.NRCARDPME    = CA.NRCARDPME)    AND (CA.CDTIPOCARD   = TC.CDTIPOCARD)   AND (CA.DTINICARDPME  = :P_DTINICARDPME) AND (CA.DTFINCARDPME = :P_DTFINCARDPME) ORDER BY TO_CHAR(CA.DTINICARDPME) DESC";

		$params = array(
	      'P_CDFILCENTPME' => $row['CENTRAL'],
	      'P_CDSERVFILI'   => $row['SERVICO'],
	      //'P_CDTIPOCARD'   => $row['TIPOCARD'],
	      'P_CDTIPOCARD'   => $row['TIPOCARD_IMPORTPRAT'],
	      'P_DTINICARDPME' => $row['PERIODODE'],
	      'P_DTFINCARDPME' => $row['PERIODOATE']
	    );
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
}