import {
  Loader, Metadata, Config, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Button, CheckboxMultiple, Select,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme08100CustoRealPeriodoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  private impTipoProduto = {
    ESTOCAVEIS: 'S',
    NAOESTOCAVEIS: 'S',
  };
  
  public clearFields() {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    unidade.setValue('');
  }
  
  public setFilterAgreg({ component }: IEventParam<Select>) {
    const nutri = Metadata.getInstance<Select>('NUTRICIONISTA');
    
    let tpCard = 'T';
    if (document.getElementsByName('TIPOCARD').length > 0) {
      const tipoCardapio = Metadata.getInstance<Select>('TIPOCARD');
      // tpCard = tipoCardapio.value.length > 0 ? tipoCardapio.value.join('_') : 'T';
      if (tipoCardapio.value.length > 0) {
        tpCard = '';
        tipoCardapio.value.forEach((item: IDictionary) => {
          tpCard += `${item.CDTIPOCARD}_`;
        });
      }
    }
      
    const conditions = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'P_CDNUTRICIO',
        operator: '=',
        value: nutri.value.length > 0 ? nutri.value.join('_') : 'T',
      },
      {
        name: 'P_CDTIPOCARD',
        operator: '=',
        value: tpCard,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      
      /* const dtFinMax = moment(row.DTDE, 'DD/MM/YYYY').add(15, 'd');
      if (row.UNIDADE.length === 0
        || (row.DTDE !== row.DTATE && moment(row.DTATE, 'DD/MM/YYYY') > dtFinMax)) {
        DialogService.show({
          name: 'confirmReport',
          type: 'warning',
          title: 'Atenção',
          text: 'Caso não seja definido opções de filtro ou informar um período superior a 15 '+
          'dias o processamento pode demorar. Deseja continuar?',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'Sim',
              events: { click: () => this.reportController.callGenerateReport(component, row, 'report/custoRealPeriodo') },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'Não',
              events: { click: () => DialogService.hide() },
            },
            {
              name: 'progressLine',
              component: 'ZdProgress',
              label: 'Progress',
              indeterminate: true,
              cssClass: 'loading-progress-line',
            },
          ],
        });
      } else { */
      this.reportController.callGenerateReport(component, row, 'report/custoRealPeriodo');
      // }
    }
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.CDTIPOCUSTO : null;
    row.NMTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.NMTIPOCUSTO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.NIVEL = row.NIVEL !== undefined ? row.NIVEL : 0;
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.ESTOCAVEIS = this.impTipoProduto.ESTOCAVEIS;
    row.NAOESTOCAVEIS = this.impTipoProduto.NAOESTOCAVEIS;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.AGRUPMASK = this.utilities.getReportMaskNivel(mask, row.NIVEL);
  }

  private formIsValid(form: IDictionary) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(dtDe, dtAte)) valid = true;
      else this.utilities.showAlert('alertError', 'Data Inicial deve ser menor que a data final.', 'red');
    }
    return valid;
  }

  public setTipoProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('ESTOCAVEIS')) this.impTipoProduto.ESTOCAVEIS = 'S';
      else this.impTipoProduto.ESTOCAVEIS = 'N';

      if (items.includes('NAOESTOC')) this.impTipoProduto.NAOESTOCAVEIS = 'S';
      else this.impTipoProduto.NAOESTOCAVEIS = 'N';
    }
  }
}
